from program import Program
from time import sleep

class Batch(object):
    
    def __init__(self, app='', prof=[], preRun=[], postRun=''):
        if app:
            self.app = app
        else:
            self.app = Program(title='default', name='iozone', args='-a')
            
        if prof and isinstance(prof, list):
            self.profList = prof
        elif prof and isinstance(prof, Program):
            self.profList = [prof]
        else:
            self.prof = [Program()]
            
        if preRun and isinstance(preRun, list):
            self.preRunList = preRun
        elif preRun and isinstance(preRun, Program):
            self.preRunList = [preRun]
        else:
            self.preRunList = []

        self.postRun = postRun
        self.title = app.title
            
    def run(self):
        for preRun in self.preRunList:
            preRun.start()
            preRun.wait()

        for prof in self.profList:
            prof.start()        

        sleep(0.5)
        self.app.start()
        sleep(0.1)
        self.app.wait() # wait for app to finish
        
        # now stop the profilers
        for prof in self.profList:
            prof.stop()
        
        results = {}
        results[self.app.name] = self.app.getResults()
        for prof in self.profList:
            results[prof.name] = prof.getResults()
        results['title'] = self.title
        
        if self.postRun:
            self.postRun.start()
            self.postRun.wait()            
        
        return results

if __name__ == '__main__':
    ssh = {'ip':'localhost','user':'ben','pw':'databas3'}
    pList = [Program('test' + str(i), 'mpstat','1', ssh=ssh) for i in range(5)]
    app = Program('app', 'python', '/home/ben/Desktop/stressTest.py', ssh=ssh)
    b = Batch(app=app, prof=pList)
    batchDict = b.run()
    for k, v in batchDict.iteritems():
        print k
        print v
        print '*'*20

