import os, sys
from time import sleep, time
import signal
import subprocess as sp
from pylab import *
import traceback
import re
from error_log import logError
import matplotlib.numerix.ma as ma

def parse(data):
    dataPoint = {}
    for testName, testParams in data.iteritems():
        if testName == 'iozone':
            parseLines = testParams['retString'].split('\n')
            if not testParams['retString']:
                    dataPoint['kB'] = -1
                    dataPoint['recLen'] = -1
                    dataPoint['throughput'] = -1                
            for i, line in enumerate(parseLines):
                if line.find('KB  reclen   write rewrite') != -1:
                    data = parseLines[i+1].split()
                    dataPoint['kB'] = data[0]
                    dataPoint['recLen'] = data[1]
                    try:
                        dataPoint['throughput'] = data[2]
                    except:
                        logError(['problem parsing iozone output'])
                        dataPoint['throughput'] = -1

        if testName == 'vmstat':
            '''
            procs -----------memory---------- ---swap-- -----io---- -system-- ----cpu----
            r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa
            0  0      0 153148 231912 251608    0    0   336     3   22   83  1  2 95  2
            '''

            labels = ['r','b','swpd','free','buff','cache','si','so','bi',
                      'bo','in','cs','us','sy','id','wa']
                    
            vals = dict((label,[]) for label in labels)
            parseLines = testParams['retString'].splitlines()
            searchExpr = re.compile(r'(\d+\s*){16}')
            for line in parseLines:
                try:
                    if searchExpr.search(line):
                        tokens = line.split()
                        for i, label in enumerate(labels):
                            vals[label].append(int(tokens[i]))
                except Exception, e: 
                    logInfo = ['token error']
                    logInfo.extend(tokens)
                    logError(logInfo)

            for label in labels:
                if vals[label]: 
                    dataPoint[label] = 1.*sum(vals[label])/len(vals[label])
                else: dataPoint[label] = -1 
                
        if testName == 'mpstat':
            labels = ['user','nice','sys','iowait','irq','soft','steal',
                      'idle','intr']

            vals = dict((label,[]) for label in labels)
            parseLines = testParams['retString'].splitlines()
            for line in parseLines:
                try:
                    tokens = line.split()
                    if ('AM' in tokens or 'PM' in tokens) and 'CPU' not in tokens:
                        for i in range(9):
                            vals[labels[i]].append(float(tokens[i+3]))
                except: 
                    logInfo = ['token error']
                    logInfo.extend(tokens)
                    logError(logInfo)
                    
            for label in labels:
                if vals[label]: 
                    dataPoint[label] = 1.*sum(vals[label])/len(vals[label])
                else: dataPoint[label] = -1

        if testName == 'opcontrol':
            '''
            Counted DTLB_MISSES events (DTLB miss events) with a unit mask of 0x0f (multiple flags) count 10000
            Samples on CPU 0
            Samples on CPU 1
            samples  %        samples  %        image name               app name                 symbol name
            4762     55.7873  6578     82.4621  vmlinux                  qemu-system-x86_64       vmx_vcpu_run
            257       3.0108  81        1.0154  libc-2.7.so              qemu-system-x86_64       memcpy
            '''  
            labels = ['samples-cpu1','percent-cpu1','samples-cpu2',
                      'percent-cpu2','image','app','symbol']
                    
            vals = dict((label,[]) for label in labels)
            parseLines = testParams['retString'].splitlines()
            searchExpr =  re.compile(r'((\d+|\d+\.\d+)\s+){4}')
            for line in parseLines:
                try:
                    if searchExpr.search(line) and \
                        ('qemu-system-x86_64' in line or 'iozone' in line):
                        tokens = line.split()
                        for i, label in enumerate(labels):
                            if label.startswith('samples'):
                                vals[label].append(int(tokens[i]))
                            elif label.startswith('percent'):
                                vals[label].append(float(tokens[i]))
                            else:
                                vals[label].append(tokens[i])
                except Exception, e: 
                    logInfo = ['token error']
                    logInfo.extend(tokens)
                    logError(logInfo)

            for label in labels:
                if vals[label] and label.startswith('samples'): 
                    dataPoint[label] = sum(vals[label])
                else: dataPoint[label] = -1
                
    return dataPoint
    
def plotGraph(profileName, benchName, info, plotTitle, showFigure=False, 
              unique=False, xData=None):
    sub1 = subplot(211)
    dataLists = {}

    types = info[0].keys()
    for type in types:
        dataLists[type] = {}
        
    for dataPoint in info:
        for testType, dataDict in dataPoint.iteritems():
            for dataType, dataVal in dataDict.iteritems():
                if dataType not in dataLists[testType]: 
                    dataLists[testType][dataType] = [] 
                if dataVal is not None: dataLists[testType][dataType].append(dataVal)
                else: dataLists[testType][dataType].append(0)

    ##########################################################################
    ## IOZONE GRAPHER                                                         
    ##########################################################################
    if benchName == 'iozone':
        if xData is None: 
            xData = dataLists['virtual']['recLen']
        xMin, xMax = int(xData[0]), int(xData[-1])     #int(xData[0])
        for type in types:
            yData = array(dataLists[type]['throughput'])
            #yData = ma.masked_where(yData < 0 , yData)

            if type=='virtual': plot(xData,yData,'r')
            else: plot(xData,yData,'b')
        sub1.set_xlim(xmax=xMax, xmin=xMin)
        #sub1.set_ylim(ymax=100000)
        title(plotTitle)
        ylabel('Throughput (kBps)')
        legend(types,1)

    sub2 = subplot(212)
    
    ##########################################################################
    ## MPSTAT GRAPHER                                                         
    ##########################################################################
    if profileName == 'mpstat':
        lines = []
        if xData is None: xData = dataLists['virtual']['recLen']
        xMin, xMax = int(xData[0]), int(xData[-1]) #int(xData[0])
        #yLabels = ['user','nice','sys','iowait','irq','soft','steal']
        #yLabels = {'user':'-.','sys':'.','iowait':'^'}
        yLabels = {}
        legendList = []
        for type in types:
            for item in yLabels.keys():
                try:
                    if type == 'virtual':
                        line = plot(xData,dataLists[type][item],'r' + yLabels[item])
                    else:
                        line = plot(xData,dataLists[type][item],'b' + yLabels[item])
                    legendList.append(type + ' ' + item)
                    lines.append(line)
                except: pass
            
            yData = array(dataLists[type]['idle'])
            #yData = ma.masked_where(yData >= 95, yData)
            try:
                if type == 'virtual': line = plot(xData,100-yData,'r')
                else: line = plot(xData,100-yData,'b')
                legendList.append(type + ' total')
                lines.append(line)
            except: pass
            
        xlabel('File Size (MB)')
        ylabel('CPU Utilization (%)')
        sub2.set_xlim(xmax=xMax, xmin=xMin)
        #legend(legendList,2)
        #leg = legend(lines, legendList,'lower right')
        
        ax2 = twinx()
        yData = array(dataLists['virtual']['intr'])
        #yData = ma.masked_where(yData <= 0, yData)
        l1 = plot(xData, yData,'r--')
        yData = array(dataLists['native']['intr'])
        #yData = ma.masked_where(yData <= 0, yData)
        l2 = plot(xData, yData,'b--')
        ylabel('Interrupts per second')
        ax2.yaxis.tick_right()  
        ax2.set_xlim(xmax=xMax, xmin=xMin)  
        leg2 = legend(lines + [l1, l2],legendList + ['virtual intr/s','native intr/s'],loc='lower right') #4'''
        
        
    imgString = './graphs/' + plotTitle
    if unique: imgString += ' ' + str(int(time.time()))
    imgString += '.pdf'
    savefig(imgString, format='pdf')
    if showFigure: show()
    clf() 
                
def parseTrace(name):
    f = open(name,'r')
    recLen = 512
    xList = []
    yList = []
    timeSoFar = 0
    for i,line in enumerate(f):
        if i != 0:
            tokens = line.split()
            try:
                secLatency = int(tokens[1])/1e6
            except:
                continue
            throughput = recLen / secLatency
            timeSoFar += secLatency
            xList.append(tokens[0])
            yList.append(secLatency)
    f.close()
    #print xList, yList
    return xList, yList

def pickFile(path='./files', singleFile=False):
    fileList = sorted(os.listdir(path))
    sizeList = [os.stat(os.path.join(path, f)).st_size for f in fileList]

    print '%3s|%-65s|%9s' % ('#'.center(3),
                             'file name'.center(65),
                             'size'.center(9))
    print '-'*80
    for i, (f, s) in enumerate(zip(fileList, sizeList)):
        outStr = '%3d %-65s %7dkB' % (i+1, f[:65], s/1024)
        print outStr[:80]
    print '-'*80
    if singleFile:
        i = int(raw_input('Which file? Enter number: ')) -1
        return os.path.join(path, fileList[i])
    else:
        iList = raw_input('Which files? Enter numbers (ex. 1 3 4): ').split()
        return [os.path.join(path, fileList[int(i)-1]) for i in iList]
    
def pickKeys(d, prompt='Which to print?'):
    keyList = d.keys()
    for i, key in enumerate(keyList):
        print '%3d %-14s %s' % (i+1, key, type(d[key]))
    
    keyNums = raw_input('%s Enter numbers (ex. 1 3 4): ' % prompt).split()
    try:
        retlist = [keyList[int(i)-1] for i in keyNums]
    except Exception, e:
        print 'Invalid input. Try again'
        retlist = pickKeys(d, prompt)
    return retlist
        
def getInfo(fName):
    f = open(fName,'r')
    info = eval(f.read())
    f.close() 
    return info

def listResults(interactive=True, log=False, outfile='resultsList.txt'):
    ''' list results in table form either to stdout or to log file '''
    fName = pickFile(singleFile=True)
    info = getInfo(fName)
    keyList = pickKeys(info)
    
    if log: out = open(outfile,'a')
    
    for key in keyList:
        for x in info[key]:
            x.update(parse(x))

    key2List = pickKeys(info[keyList[0]][0])

    queryStr = 'There are %d entries. ' % len(info[keyList[0]])
    queryStr += 'Please specify range (ex. 1-200, ALL): ' 
    dataRange = raw_input(queryStr) 
    if dataRange and dataRange != 'ALL':
        dataRange = dataRange.split('-')
        start, stop = int(dataRange[0])-1, int(dataRange[1])
    else:
        start, stop = 0, len(info[keyList[0]])

    for i in xrange(start, stop):
        fmtStr = '#%-10d ' % (i + 1)
        for key2 in key2List:
            fmtStr += '%-10s ' % key2
        print fmtStr
        if log and i == 0: out.write(fmtStr + '\n')
        for key in keyList:
            dataStr = '%-10s ' % key 
            for key2 in key2List:
                if key2 in info[key][i]:
                    if isinstance(info[key][i][key2], str):
                        dataStr += '%-10s ' % info[key][i][key2][:10]
                    elif isinstance(info[key][i][key2], float):
                        dataStr += '%-10.1f ' % info[key][i][key2]
                    elif isinstance(info[key][i][key2], dict):
                        dataStr += '\n\n'
                        for k, v in info[key][i][key2].iteritems():
                            dataStr += k + '\n' + v
                    else:
                        dataStr += '%-10s ' % str(info[key][i][key2])[:10]
            print dataStr
            if log: out.write(dataStr + '\n')

        if interactive: raw_input('(enter for next data point)')
            
    if log: out.close()    
    print 'done'
    
def getMasked(vals, expr='< 0', exprList=['> 9999999']):
    vals = ma.array(vals)
    execStr = 'vals ' + expr
    for e in exprList:
        execStr += ' or vals ' + e
    return ma.masked_where(eval(execStr), vals)
    
def plotResults():
    ''' subplot is numrows, numcols, axesnum '''
    names = pickFile()

    subplots = int(raw_input('How many subplots? '))
    rangeBool = raw_input('Specify custom x range (y/n)? ') == 'y'
    infoList = []
    lineList = [[] for x in range(subplots)]
    legendList = [[] for x in range(subplots)]
    xyList = [[] for x in range(subplots)]
    for name in names:
        print 'Setting up %s...' % name
        info = getInfo(name)

        doAgain = True
        while doAgain:
            if subplots != 1: 
                subplotNum = int(raw_input('Which subplot? (-1 to quit) '))
                if subplotNum == -1:
                    break
                subplot(subplots*100 + 10 + subplotNum)
            else:
                subplotNum = 1  
                
            keyList = pickKeys(info)
            
            for key in keyList:
                for x in info[key]:
                    x.update(parse(x))
            
            paramList = pickKeys(info[keyList[0]][0], prompt='x y1 y2 y3 ...?') 
            xParam = paramList[0]
            yParamList = paramList[1:]
            xlabel(xParam)
            ylabel(str(yParamList))

            xList = []
            for x in info[keyList[0]]:
                try:
                    xList.append(int(x[xParam]))
                except Exception, e:
                    print x

            minX = min([x for x in xList if x > 0])
            maxX = max(xList)
            queryStr = 'x ranges from %d to %d. ' % (minX, maxX)
            if rangeBool:
                queryStr += 'Please specify x-axis range (ex. 0-200, ALL): ' 
                dataRange = raw_input(queryStr)
            else:
                print queryStr
                dataRange = None
                
            if dataRange and dataRange != 'ALL':
                try:
                    dataRange = dataRange.split('-')
                    minX, maxX = int(dataRange[0]), int(dataRange[1])
                except Exception, e:
                    dataRange = dataRange.split()
                    minX, maxX = int(dataRange[0]), int(dataRange[1])               

            for key in keyList:
                for yParam in yParamList:
                    yList = [float(y[yParam]) for y in info[key]]
                    line = plot(getMasked(xList),getMasked(yList), lw=2.0)
                    xyList[subplotNum-1].append((getMasked(xList),getMasked(yList)))
                    lineList[subplotNum-1].append(line)
                    legendList[subplotNum-1].append('%s %s %s' % (name[8:].split('-')[0], key, yParam))
            
            axis(xmin=minX, xmax=maxX)
            if subplots == 1: doAgain = raw_input('Another plot from same file (y/n)? ') == 'y'
    print 'subplots:', subplots
    for i in range(subplots):
        subplot(subplots*100 + 10 + i + 1)        
        legend(lineList[i], legendList[i]) #loc='lower right'

    show()

    linewidth = 0
    while raw_input('Plot again? (y/n) ') == 'y': 
        if raw_input('Change configuration? (y/n) ') == 'y':
            inpChoice = 1
            while inpChoice != '0':
                print '1 - change legend location'
                print '2 - change line color'
                print '3 - add another subplot'
                print '4 - change x range'
                print '5 - change y range'
                print '6 - change labels'
                print '7 - change line widths'
                print '0 - quit'
                inpChoice = raw_input('>> ')
                if inpChoice == '7':
                    lineWidth = float(raw_input('Input line width (float): '))
        for i in range(subplots):
            subplot(subplots*100 + 10 + i + 1)   
            for x, y in xyList[i]:
                if lineWidth:
                    plot(x, y, lw=lineWidth)
                else:
                    plot(x, y, lw=2.0)
            legend(lineList[i], legendList[i]) #loc='lower right'
            axis(xmin=minX, xmax=maxX)

        xlabel(xParam)
        ylabel(str(yParamList))
        show()
 
if __name__ == '__main__':
    
    choice = int(raw_input('1 - Plot\n2 - Print\n'))
    if choice == 1:
        plotResults()
    else:
        interactive = raw_input('Interactive? (y/n) ') == 'y'
        log = raw_input('Log results? (y/n) ' ) == 'y'
        listResults(interactive=interactive, log=log)
    
'''
1. do references
2. do kvm, xen figures
3. finish smaller blips that need to be re-written
4. do related work, conclusion
5. do mine figure
'''