#!/usr/bin/python

import socket
import md5
import time
import os

class PopenRemote(object):
    def __init__(self, title='test', name='vmstat', 
                 args='1', ip='127.0.0.1'):
        self.port = 60001
        self.bufSize = 1024
        self.sock = self.connect(ip)
        self.sendHash('password')
        self.pid = self.run(name, args)

    def connect(self, ip, verbose=True):        
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if verbose: print 'Trying to connect...'
        while True:
            try:
                sock.connect((ip,self.port))
                break
            except socket.error, e:
                if verbose: 
                    print 'No server found. Retrying...'
                    print str(e)
                    time.sleep(1)
        return sock

    def sendHash(self, pwd):
        md5Hash = md5.new(pwd).hexdigest()
        self.sendLine(md5Hash)
        print self.getLine()

    def sendLine(self, cmd):
        self.sock.send(cmd)

    def run(self, name, args):
        runStr = ' '.join(('open', name))
        if args:
            runStr = ' '.join((runStr, args))
        self.sendLine(runStr)
        return self.getLine()
                        
    def sendCmd(self, cmd):
        print 'sent:', cmd + ' ' + self.pid
        self.sendLine(cmd + ' ' + self.pid)
        return self.getLine()

    def kill(self):
        return self.sendCmd('kill')

    def wait(self):
        return self.sendCmd('wait')

    def getLine(self):
        return self.sock.recv(self.bufSize).strip()
    
    def readStdout(self, blocking=True):
        if blocking:
            strReturn = self.sendCmd('stdout')
            while strReturn == '':
                strReturn = self.sendCmd('stdout')
                time.sleep(1)  
            return strReturn
        else:
            return self.sendCmd('stdout')

    def readStderr(self, blocking=False):
        if blocking:
            strReturn = self.sendCmd('stderr')
            while strReturn != '':
                 strReturn = self.sendCmd('stderr')
            return strReturn
        else:
            return self.sendCmd('stderr')

if __name__ == '__main__':
    prog = PopenRemote(title='test', name='python', 
                       args='stressTest.py', ip='home-main')
    data = prog.readStdout(blocking=False)
    while data != '':
        print data
        data = prog.readStdout(blocking=False)
    prog.kill()
    prog.wait()
    print 'done'
    prog.sock.close()
    
