#!/usr/bin/python

import os
import signal
from subprocess import Popen, PIPE
import time
import threading
import Queue

class Process:
    def __init__(self, cmd, stdin=False):
        self.threading = True
        if stdin:
            self._proc = Popen('exec '+cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
            self._stdin = Queue.Queue(0)
            threading.Thread(target=self._inThread).start()
        else:
            self._proc = Popen('exec '+cmd, shell=True, stdout=PIPE, stderr=PIPE)
            self._stdin = None
            
        self._stderr = Queue.Queue(0)
        self._stdout = Queue.Queue(0)
        threading.Thread(target=self._outThread).start()
        threading.Thread(target=self._errThread).start()
        self.pid = self._proc.pid
    
    def kill(self):
        os.kill(self.pid, signal.SIGKILL)
        self.wait()
    
    def wait(self):
        self._proc.wait()
        self.threading = False
    
    def write(self, msg):
        self._stdin.put(msg)
    
    def empty(self, device):
        if device == 'stdout':
            return self._stdout.empty()
        elif device == 'stderr':
            return self._stderr.empty()
    
    def readline(self, device, block=True):
        if device == 'stdout':
            try:
                return self._stdout.get(block=block)
            except Queue.Empty():
                return None
        elif device == 'stderr':
            try:
                return self._stderr.get(block=block)
            except Queue.Empty():
                return None
    
    def _inThread(self):
        while self.threading:
            if not self._stdin.empty():
                self._proc.stdin.write(self._stdin.get())
    
    def _outThread(self):
        while self.threading:
            self._stdout.put(self._proc.stdout.readline().strip())
        
    def _errThread(self):
        while self.threading:
            self._stderr.put(self._proc.stderr.readline().strip())

if __name__=='__main__':
    proc = Process('python stressTest.py')
    time.sleep(60*10)
    #for i in range(1000000):
        #print proc.readline('stdout')
        #print proc.readline('stderr')
    proc.kill()

