import os, sys
from time import sleep, time
from signal import SIGTERM
from threading import Thread
import subprocess as sp
from pxssh import pxssh as SSHConnection
from pylab import *
from re import search

from error_log import logError

class Program(Thread):
    
    def __init__(self, title='prog', name='mpstat', 
                 args='1 999999', ssh={}, verbose=True, stopCmd=[]):
        Thread.__init__(self)
        if ssh:
            self.conn = SSHConnection()
            self.ip = ssh['ip']
            self.user = ssh['user']
            self.pw = ssh['pw']
        else:
            self.conn = None
        self.cmdString = name + ' ' + args
        self.title = title
        self.name = name
        self.args = args
        self.verbose = verbose
        self.retString = ''
        self.stopCmd = stopCmd
        
    def wait(self):
        while self.isAlive(): 
            self.join(timeout=60*60)
        if self.conn:
            self._close()
        
    def stop(self):
        if self.conn:
            newConn = SSHConnection()
            self._connect(conn=newConn)
            if not self.stopCmd:
                res = self._runRemote('killall ' + self.name, conn=newConn)
                while search('no process killed',res):
                    print 'hi remote'
                    res = self._runRemote('killall ' + self.name, conn=newConn)  
            else:
                for sc in self.stopCmd:
                    self.retString += '\n' + self._runRemote(sc, conn=newConn)
            self._close(conn=newConn)
        else:
            if not self.stopCmd:
                pids = self._runLocal('pidof ' + self.name).split()
                while not pids:
                    pids = self._runLocal('pidof ' + self.name).split()
                    print 'hi'
                for pid in pids:
                    try:
                        os.kill(int(pid), SIGTERM) 
                    except Exception:
                        logError(['os kill error', self.cmdString])
            else:
                for sc in self.stopCmd:
                    self.retString += '\n' + self._runLocal(sc)
        self.join()
                
    def getResults(self, tup=False):
        if tup:
            return self.name, self.args, self.retString
        else:
            res = {}
            res['name'] = self.name
            res['retString'] = self.retString
            res['args'] = self.args
            return res
        
    def _connect(self, conn=None):
        if not conn:
            conn = self.conn
        while True:
            try:
                conn.login(self.ip, self.user, self.pw)
                break
            except Exception: 
                logError(['ssh login exception', self.cmdString])
                sleep(0.5)   

    def run(self):
        if self.conn:
            if self.verbose: 
                print 'starting remote connection for %s' % self.title
                print 'cmd: %s' % self.cmdString
            self._connect()
            self.retString = self._runRemote()
            self._close()
        else:
            if self.verbose: 
                print 'starting local connection for %s' % self.title
                print 'cmd: %s' % self.cmdString
            self.retString = self._runLocal()
        if self.verbose: 
            print 'closing connection for %s' % self.title
    
    def _close(self, conn=None):
        if not conn:
            conn = self.conn
        try:
            conn.logout()
        except Exception: 
            logError(['ssh logout exception', self.cmdString])
            sleep(0.5) 
        
    def _runLocal(self, cmd=None):
        ''' method to run a command over the local shell. '''
        if cmd:
            prog = sp.Popen(cmd, shell=True, stdout=sp.PIPE, stdin=sp.PIPE)
        else:
            prog = sp.Popen(self.cmdString, shell=True,
                            stdout=sp.PIPE, stdin=sp.PIPE)
        return prog.communicate()[0]
        
    def _runRemote(self, cmd=None, conn=None, timeout=999999):
        res = ''
        if not cmd: 
            cmd = self.cmdString
        if not conn:
            conn = self.conn
        try:
            conn.sendline(cmd)
            conn.prompt(timeout=timeout) # don't want to timeout here
            res = conn.before 
        except Exception:
            logError(['ssh remote command exception', cmd])
            sleep(0.5)   
        
        return res
    
if __name__ == '__main__':
    ssh = {'ip':'localhost','user':'ben','pw':'databas3'}
    p = Program(title='prog', name='python', args='/home/ben/Desktop/stressTest.py', ssh=None)
    p.start()
    print 'hello world'
    sleep(2)
    p.stop()
    print 'Results', p.getResults()
